// Code4B: Floyd-Warshall's Method
#include "Code4B.h"

BEGIN_MESSAGE_MAP(CCode4B, CFrameWnd)
    ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_BN_CLICKED(IDC_NGRAPH,OnNewGraph)
END_MESSAGE_MAP()

CCode4B::CCode4B()
{
    Create(NULL,L"Floyd-Warshall's Shortest Path Algorithm",
		WS_OVERLAPPEDWINDOW,CRect(0,0,1000,800));
	bNGraph.Create(L"New Graph",WS_CHILD | WS_VISIBLE | BS_DEFPUSHBUTTON,
		CRect(CPoint(30,30),CSize(180,40)),this,IDC_NGRAPH);
	home=CPoint(30,100); end=CPoint(500,700);
	fArial.CreatePointFont(60,L"Arial");
	fHelvetica.CreatePointFont(100,L"Helvetica");
	Initialize();
}

void CCode4B::OnLButtonDown(UINT nFlags,CPoint pt)
{
	for (int i=1;i<=N;i++)
		if (v[i].rct.PtInRect(pt))
		{
			bFlag++;
			if (bFlag==1)
			{
				Sv=i; 
				DrawNode(Sv);
			}
			if (bFlag==2)
			{
				Dv=i;
				DrawNode(Dv);
				Compute();
				bFlag=0;
			}
		}
}

void CCode4B::ShowTable()
{
	CString s;
	int i,j;
	CRect rct;
	table1.DestroyWindow(); table2.DestroyWindow();
	// creates the list view window
	rct=CRect(CPoint(end.x+130,home.y-30),CSize(330,300));
   	table1.Create(WS_VISIBLE | WS_CHILD | WS_DLGFRAME | LVS_REPORT
       		 | LVS_NOSORTHEADER,rct,this,802);
	rct=CRect(CPoint(end.x+130,home.y+320),CSize(330,300));
   	table2.Create(WS_VISIBLE | WS_CHILD | WS_DLGFRAME | LVS_REPORT
       		 | LVS_NOSORTHEADER,rct,this,803);


	for (j=0;j<=N;j++)
	{
		s.Format(L"%d",j);
		table1.InsertColumn(j,((j==0)?L"SP":s),LVCFMT_CENTER,30);
		table2.InsertColumn(j,((j==0)?L"wt":s),LVCFMT_CENTER,30);
	}
	for (i=1;i<=N;i++)
	{
		s.Format(L"%d",i); 
		table1.InsertItem(i-1,s,0);
		table2.InsertItem(i-1,s,0);
		for (j=1;j<=N;j++)
		{
			s.Format(L"%d",v[i].sp[j]); table1.SetItemText(i-1,j,s);
			s.Format(L"%d",v[i].wt[j]); table2.SetItemText(i-1,j,s);
		}
	}
}

void CCode4B::Initialize()
{	
	int i,j,k;
	double distance;
	bFlag=0;
	srand(time(0));
	for (i=1;i<=N;i++)
	{
		v[i].home.x=home.x+20+rand()%(end.x-home.x-50);
		v[i].home.y=home.y+20+rand()%(end.y-home.y-50);
		v[i].rct=CRect(v[i].home.x,v[i].home.y,v[i].home.x+25,v[i].home.y+25);
	}	
	for (i=1;i<=N;i++)
	{
		v[i].wt[i]=0;
		for (j=i+1;j<=N;j++)
		{
			distance=sqrt(pow((double)(v[i].home.x-v[j].home.x),2)
							+pow((double)(v[i].home.y-v[j].home.y),2));
			if (distance<=LinkRange)
				v[i].wt[j]=1+rand()%9;
			else
				v[i].wt[j]=99;
			v[j].wt[i]=v[i].wt[j];
		}
	}
	for (i=1;i<=N;i++)
        for (j=1;j<=N;j++)
		{
            v[i].sp[j]=v[i].wt[j];
			for (k=1;k<=N;k++)
				alpha[i][j][k]=99;
		}
	FloydWarshall(); 
	ShowTable();
}

void CCode4B::OnPaint()
{
	CPaintDC dc(this);
	CString s;
	CRect rct;
	CPen rPen(PS_SOLID,3,RGB(0,0,0));
	CPen mPen(PS_SOLID,2,RGB(0,0,0));
	CPen qPen(PS_SOLID,1,RGB(100,100,100));
	CPoint mPoint,pt;
	int i,j;
	dc.SelectObject(rPen); 
	dc.Rectangle(home.x-10,home.y-10,end.x+10,end.y+10);
	dc.SelectObject(qPen); dc.SelectObject(fArial);
	for (i=1;i<=N;i++)
		for (j=1;j<=N;j++)
			if (v[i].wt[j]!=99)
			{
				dc.MoveTo(CPoint(v[i].home)); 
				dc.LineTo(CPoint(v[j].home));
				mPoint=CPoint((v[i].home.x+v[j].home.x)/2,
					(v[i].home.y+v[j].home.y)/2);
				s.Format(L"%d",v[i].wt[j]);
				dc.TextOut(mPoint.x,mPoint.y,s);
			}
	dc.SelectObject(fHelvetica); dc.SelectObject(mPen);
	for (i=1;i<=N;i++)
		DrawNode(i);
	dc.TextOutW(home.x+70,end.y+20,L"Click any two nodes to see the shortest path");
}

void CCode4B::FloydWarshall()
{
    int i,j,k,m,r;    
    for (i=1;i<=N;i++)
        for (j=1;j<=N;j++)
            for (k=1;k<=N;k++)
                if (v[j].sp[i]!=99 || v[i].sp[k]!=99 || v[j].sp[k]!=99)
                    if (v[j].sp[i]+v[i].sp[k]<v[j].sp[k])
                    {		
						r=1;
                        v[j].sp[k]=v[j].sp[i]+v[i].sp[k];
						for (m=1;m<=N;m++)
							alpha[j][k][m]=99;
						for (m=1; m<=N; m++)
							if (alpha[j][i][m]!=99)
								alpha[j][k][r++]=alpha[j][i][m];
						alpha[j][k][r++]=i;
						for (m=1; m<=N; m++)
							if (alpha[i][k][m]!=99)
								alpha[j][k][r++]=alpha[i][k][m];
					}
}

void CCode4B::Compute()
{	
    CClientDC dc(this);
	CString s;
	CPen pBlue(PS_SOLID,3,RGB(0,0,200));
	int k,u,w,r;
	dc.SelectObject(fHelvetica);
	s.Format(L"Total Costs: %d",v[Sv].sp[Dv]); 
	dc.TextOut(end.x+20,home.y+10,s);

	r=1;
	DrawVCPath(r,Sv,0);
    dc.SelectObject(&pBlue);
    w=Sv; dc.MoveTo(v[w].home);
	for (k=1; k<=N; k++)		// display the vias
	{
		u=alpha[Sv][Dv][k];
		if (u!=99)
		{
			r++; e[Sv][Dv].via[r]=u;
			dc.LineTo(v[u].home);
			DrawVCPath(r,u,w);
			w=u; 
		}
	}
	if (v[Dv].sp[w]!=99)		// display the destination
		dc.LineTo(v[Dv].home);
	e[Sv][Dv].via[r]=Dv; 
	DrawVCPath(r+1,Dv,w);
	DrawNode(Sv); DrawNode(Dv);
}

void CCode4B::DrawVCPath(int r,int u,int w)
{
	CClientDC dc(this);
	CString s;
	CPoint pt;
	CRect rct;
	pt=CPoint(end.x+60,home.y+50+60*(r-1));
	rct=CRect(pt.x-5,pt.y-5,pt.x+20,pt.y+20); dc.Rectangle(rct);
	dc.SelectObject(fHelvetica);
	s.Format(L"%d",u);
	dc.TextOut(pt.x,pt.y,s);
	dc.SelectObject(fArial);
	if (r>1)
	{		
		s.Format(L"%d",v[u].wt[w]);
		dc.TextOut(end.x+60,home.y+20+60*(r-1),s);
	}
}

void CCode4B::DrawNode(int u)
{	
	CClientDC dc(this);
	CString s;
	CPoint pt;
	CPen pBlack(PS_SOLID,2,RGB(0,0,0));
	CPen pRed(PS_SOLID,3,RGB(200,0,0));
	if (bFlag==0)
		dc.SelectObject(pBlack);
	else
		dc.SelectObject(pRed);
	pt=v[u].home;
	dc.Rectangle(v[u].rct);
	s.Format(L"%d",u); dc.TextOut(pt.x+5,pt.y+5,s);
}

void CCode4B::OnNewGraph()
{
	Initialize(); Invalidate();
}