// Code4C.h
#include "Code4C.h"

BEGIN_MESSAGE_MAP(CCode4C, CFrameWnd)
    ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_BN_CLICKED(IDC_NGRAPH,OnNewGraph)
END_MESSAGE_MAP()

CCode4C::CCode4C()
{
	home=CPoint(30,100); end=CPoint(500,700);
	Create(NULL, L"Mini GPS",WS_OVERLAPPEDWINDOW,CRect(0,0,1100,800));
	bNGraph.Create(L"New Graph",WS_CHILD | WS_VISIBLE | BS_DEFPUSHBUTTON,
		CRect(CPoint(home.x,home.y-60),CSize(180,40)),this,IDC_NGRAPH);
	DisplaySetup(); OnNewGraph();
}

void CCode4C::OnNewGraph()
{	
	CString s[]={"Bus","S","D","SP","N","Path"};
	nBus=0; Initialize(); Invalidate();	
	table.DestroyWindow();
	table.Create(WS_VISIBLE | WS_CHILD | WS_BORDER | LVS_REPORT 
		| LVS_NOSORTHEADER,CRect(CPoint(end.x+30,home.y),CSize(390,250)),
		this,IDC_TABLE);
	for (int i=0;i<=5;i++)
		table.InsertColumn(i,s[i],LVCFMT_LEFT,((i==5)?250:40));
}

void CCode4C::DisplaySetup()
{	
	cColor=1;
	int Color[]={RGB(150,150,150),
		RGB(0,0,200),RGB(200,0,0),
		RGB(0,200,0),RGB(200,200,0),
		RGB(0,200,200),RGB(200,0,200),
		RGB(200,50,150),RGB(100,100,255),
		RGB(250,50,255),RGB(100,200,50)};
	for (int i=0;i<=10;i++)
		pBus[i].CreatePen(PS_SOLID,2,Color[i]);
	fHelvetica.CreatePointFont (100,L"Helvetica");
	fArial.CreatePointFont (60,L"Arial");
}

void CCode4C::Initialize()
{
	int i,j,u,w;
	double distance;
	srand(time(0));
	bFlag=0;			
	if (nBus==0)
	{				
		for (i=1;i<=N;i++)
		{		
			v[i].home.x=home.x+20+rand()%(end.x-home.x-50);
			v[i].home.y=home.y+20+rand()%(end.y-home.y-50);
			v[i].rct=CRect(v[i].home.x,v[i].home.y,v[i].home.x+25,v[i].home.y+25);
		}
		for (i=1;i<=N;i++)
			for (j=i;j<=N;j++)
			{
				v[i].wt[j]=99;
				distance=sqrt(pow((double)(v[i].home.x-v[j].home.x),2)
					+pow((double)(v[i].home.y-v[j].home.y),2));
				if (distance<LinkRange)
					v[i].wt[j]=1+rand()%9;
				v[j].wt[i]=v[i].wt[j];
			}
	}
	else
	{
		u=Bus[nBus].Path[1];
		for (i=1;i<=Bus[nBus].nNodes-1;i++)
		{
			w=Bus[nBus].Path[i+1];
			v[u].wt[w]=99; v[w].wt[u]=99;
			u=w;
		}
	}
}

void CCode4C::DrawNode(int u)
{	
	CClientDC dc(this);
	CString s;
	CPoint pt;
	CPen pBlack(PS_SOLID,2,RGB(0,0,0));
	CPen pRed(PS_SOLID,3,RGB(200,0,0));
	if (u==Sv || u==Dv)
		dc.SelectObject(pRed);
	else
		dc.SelectObject(pBlack);
	pt=v[u].home;
	dc.Rectangle(v[u].rct);
	s.Format(L"%d",u); dc.TextOut(pt.x+5,pt.y+5,s);
}

void CCode4C::OnPaint()
{
	CPaintDC dc(this);
	CString s;
	CRect rct;
	CPen rPen(PS_SOLID,3,RGB(0,0,0));
	CPen mPen(PS_SOLID,2,RGB(0,0,0));
	CPen qPen(PS_SOLID,1,RGB(100,100,100));
	CPoint mPoint,pt;
	int i,j;
	dc.SelectObject(rPen); 
	dc.Rectangle(home.x-10,home.y-10,end.x+10,end.y+10);
	dc.SelectObject(qPen); dc.SelectObject(fArial);
	for (i=1;i<=N;i++)
		for (j=1;j<=N;j++)
			if (v[i].wt[j]!=99)
			{
				dc.MoveTo(CPoint(v[i].home)); 
				dc.LineTo(CPoint(v[j].home));
				mPoint=CPoint((v[i].home.x+v[j].home.x)/2,
					(v[i].home.y+v[j].home.y)/2);
				s.Format(L"%d",v[i].wt[j]);
				dc.TextOut(mPoint.x,mPoint.y,s);
			}
	dc.SelectObject(fHelvetica); dc.SelectObject(mPen);
	for (i=1;i<=N;i++)
		DrawNode(i);
}

int CCode4C::GetCUN()
{
	int minDistance=99;
	int cun;
	for (int i=1;i<=N;i++)
		if ((!fv[i]) && (minDistance>=v[Sv].sp[i]))
		{
			minDistance=v[Sv].sp[i];
			cun=i;
		}
	return cun;
}

void CCode4C::Dijkstra()
{
	int minDistance=99;
	int cun;

	// initialize
	for (int i=1;i<=N;i++)
	{
		fv[i]=false;
		Pv[i]=-1;
		v[Sv].sp[i]=99;
	}
	v[Sv].sp[Sv]=0;

	// compute	
	int k=0;
	while (k<N)
	{
		cun=GetCUN();
		fv[cun]=true;
		for (int i=1;i<=N;i++)
			if ((!fv[i]) && (v[cun].wt[i]>0))
				if (v[Sv].sp[i]>v[Sv].sp[cun]+v[cun].wt[i])
				{
					v[Sv].sp[i]=v[Sv].sp[cun]+v[cun].wt[i];
					Pv[i]=cun;
				}
		k++;
	}	
	nBus++;
	Bus[nBus].sp=v[Sv].sp[Dv];
}

void CCode4C::DrawPath()
{ 
	CClientDC dc(this);
	int w,i,u,r,dummy[N+1];

	// display the source
	dc.SelectObject(pBus[1+cColor%10]); cColor++;	
	w=Dv;	Bus[nBus].nNodes=0; Bus[nBus].Path[1]=Sv;
	for (i=1;i<=N;i++)
	{
		u=Pv[w]; 
		if (u!=-1)
		{
			r=++Bus[nBus].nNodes; dummy[r]=w;
			w=u;
		}
		dummy[r+1]=Sv;
	}
	r=++Bus[nBus].nNodes; Bus[nBus].Path[r]=Dv;

	u=Sv; Bus[nBus].Path[1]=u;
	dc.MoveTo(v[u].home); 
	for (i=1;i<=Bus[nBus].nNodes;i++)
	{
		w=dummy[Bus[nBus].nNodes-i+1]; Bus[nBus].Path[i]=w;
		dc.LineTo(v[w].home); 
		u=w;
	}
	w=Dv; Bus[nBus].Path[i]=w;
	dc.LineTo(v[w].home);
}

void CCode4C::OnLButtonDown(UINT nFlags,CPoint pt)
{
	CClientDC dc(this);
	CString s;
	int i,j;
	dc.SelectObject(pBus[1]); dc.SelectObject(fArial);
	for (int i=1;i<=N;i++)
		if (v[i].rct.PtInRect(pt))
		{
			bFlag++;
			if (bFlag==1)		// bFlag=1 is the source
			{
				Sv=i;
				DrawNode(Sv);
			}
			if (bFlag==2)		// bFlag=2 is the destination
			{
				
				Dv=i; DrawNode(Dv);
				for (j=1;j<=3;j++)
				{
					Dijkstra();
					if (v[Sv].sp[Dv]!=99)
					{
						DrawPath(); ShowTable(); Initialize();
					}
					else
					{
						bFlag=0;
						dc.SelectObject(pBus[0]);
						DrawNode(Sv); DrawNode(Dv);
					}
				}
			}
		}
}

void CCode4C::ShowTable()
{
	int i,k,w,u;
	CString s,S;
	for (i=1;i<=nBus;i++)
	{
		table.DeleteItem(i-1);
		s.Format(L"%d",i); table.InsertItem(i-1,s,0);
		s.Format(L"%d",Bus[i].Path[1]); table.SetItemText(i-1,1,s);
		w=Bus[i].nNodes;
		s.Format(L"%d",Bus[i].Path[w]); table.SetItemText(i-1,2,s);
		u=Bus[i].Path[1]; w=Bus[i].Path[w];
		s.Format(L"%d",Bus[i].sp); table.SetItemText(i-1,3,s);
		s.Format(L"%d",Bus[i].nNodes); table.SetItemText(i-1,4,s);
		S="";
		for (k=1;k<=Bus[i].nNodes;k++)
		{
			s.Format(L"%d ",Bus[i].Path[k]); S += s;
		}
		table.SetItemText(i-1,5,S);
	}
}