// Code4D.cpp: Multi-stage Interconnection network
#include "Code4D.h"

BEGIN_MESSAGE_MAP(CCode4D, CFrameWnd)
    ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_BN_CLICKED (IDC_RESET,OnReset)
	ON_BN_CLICKED (IDC_START,OnStart)
END_MESSAGE_MAP()

CCode4D::CCode4D()
{		
	idc=550; flag=0; home=CPoint(10,20);
	Create(NULL, L"Multi-column Interconnection Network",
		WS_OVERLAPPEDWINDOW,CRect(0,0,1100,800));	
	sNy.Create(L"Ny (1-10)",WS_VISIBLE | SS_CENTERIMAGE | SS_CENTER,
		CRect(CPoint(670,100),CSize(100,30)),this,IDC_STATIC);
	sNx.Create(L"Nx (1-10)", WS_VISIBLE | SS_CENTERIMAGE | SS_CENTER,
		CRect(CPoint(670,140),CSize(100,30)),this,IDC_STATIC);
	eNy.Create(WS_CHILD | WS_VISIBLE | WS_BORDER | SS_CENTER, 
		CRect(CPoint(780,100),CSize(70,30)),this,idc++);
	eNx.Create(WS_CHILD | WS_VISIBLE | WS_BORDER | SS_CENTER, 
		CRect(CPoint(780,140),CSize(70,30)),this,idc++);
	Start.Create(L"Start",WS_CHILD | WS_VISIBLE | BS_DEFPUSHBUTTON, 
		CRect(CPoint(670,180),CSize(180,30)),this,IDC_START);
}

void CCode4D::OnStart()
{
	CString s;
	eNy.GetWindowText(s); Ny=_ttoi(s);
	eNx.GetWindowText(s); Nx=_ttoi(s);
	n=Nx*Ny;
	if (n<=NMax)
	{		
		sNy.DestroyWindow(); sNx.DestroyWindow(); 
		eNy.DestroyWindow(); eNx.DestroyWindow();
		Reset.Create(L"Reset",WS_CHILD | WS_VISIBLE | BS_DEFPUSHBUTTON,
			CRect(CPoint(670,10),CSize(190,30)),this,IDC_RESET);
		flag=1; DisplaySetup(); OnReset();
	}
}

void CCode4D::OnReset()
{
	nBus=0;	Initialize(); Invalidate();
	
	// create the table
	CString s[]={"Bus","S","D","SP","n","Path"};
	table.DestroyWindow();
	table.Create(WS_VISIBLE | WS_CHILD | WS_BORDER | LVS_REPORT 
		| LVS_NOSORTHEADER,CRect(CPoint(670,50),CSize(390,250)),
		this,IDC_TABLE);
	for (int i=0;i<=5;i++)
		table.InsertColumn(i,s[i],LVCFMT_LEFT,((i==5)?250:40));
}

void CCode4D::DisplaySetup()
{	
	cColor=1;
	int Color[]={RGB(150,150,150),
		RGB(0,0,200),RGB(200,0,0),
		RGB(0,200,0),RGB(200,200,0),
		RGB(0,200,200),RGB(200,0,200),
		RGB(200,100,150),RGB(100,100,255),
		RGB(250,50,255),RGB(255,255,255)};
	pBus[0].CreatePen(PS_SOLID,1,Color[0]);
	for (int i=1;i<=9;i++)
		pBus[i].CreatePen(PS_SOLID,2,Color[i]);
	fTimes.CreatePointFont (80,L"Times New Roman");
	fCourier.CreatePointFont (60,L"Courier");
	fArial.CreatePointFont (60,L"Arial");
}

void CCode4D::Initialize()
{
	int i,j,k,u,w;
	LineFlag=0;			
	if (nBus==0)
	{	
		// label the nodes		
		k=0;
		for (j=1;j<=Ny;j++)
		{
			k++;
			v[k].home=CPoint(80+home.x,20+home.y+60*(j-1));
			v[k].rct=CRect(CPoint(v[k].home.x-20,
				v[k].home.y-20),CSize(40,40));
			v[k+Ny].home=CPoint(80+home.x+60*(Nx-1),
				20+home.y+60*(j-1));
			v[k+Ny].rct=CRect(CPoint(v[k+Ny].home.x-20,
				v[k+Ny].home.y-20),CSize(40,40));
		}
		k=2*Ny;
		for (i=2;i<=Nx-1;i++)
			for (j=1;j<=Ny;j++)
			{
				k++;
				v[k].home=CPoint(80+home.x+60*(i-1),
					20+home.y+60*(j-1));
				v[k].rct=CRect(CPoint(v[k].home.x-20,
					v[k].home.y-20),CSize(40,40));
			}

		// draw edges between nodes
		for (i=1;i<=n;i++)
		{
			v[i].wt[i]=99;
			for (j=i+1;j<=n;j++)
			{
				v[i].wt[j]=99; v[j].wt[i]=99;
				if (i>2*Ny)		// middle columns
				{
					if (j-i==1 && i%Ny!=0)
					{
						v[i].wt[j]=1; v[j].wt[i]=1;
					}
					if (j-i==Ny)
					{
						v[i].wt[j]=1; v[j].wt[i]=1;
					}
				}
				if (i<=Ny && j-i==2*Ny)
				{
					v[i].wt[j]=1; v[j].wt[i]=1;
				}
				if ((i>Ny && i<=2*Ny) && j-i==(Nx-2)*Ny)
				{
					v[i].wt[j]=1; v[j].wt[i]=1;
				}
			}
		}
	}
	else
	{
		u=Bus[nBus].Path[1];
		for (i=1;i<=Bus[nBus].nNodes-1;i++)
		{
			w=Bus[nBus].Path[i+1];
			v[u].wt[w]=99; v[w].wt[u]=99;
			u=w;
		}
	}
}

void CCode4D::OnPaint()
{	
	int i,j,k;
	CPaintDC dc(this);
	CString s;
	CPen gPen(PS_SOLID,1,RGB(150,150,150));
	CPen hPen(PS_SOLID,1,RGB(0,0,200));
	if (flag)
	{
		dc.SelectObject(gPen); dc.SelectObject(fArial);

		for (i=1;i<=n;i++)
			for (j=i;j<=n;j++)
				if (v[i].wt[j]!=99)
				{
					dc.MoveTo(v[i].home); 
					dc.LineTo(v[j].home);
				}
		dc.SelectObject(&fArial); 
		dc.SetTextColor(RGB(0,0,255));
		for (k=1;k<=n;k++)
			if (k<=2*Ny)
			{

				dc.SelectObject(gPen);
				dc.Rectangle(v[k].rct); 
				s.Format(L"%d",k); 
				dc.TextOut(v[k].home.x-15,v[k].home.y-18,s);
			}
			else
			{
				dc.SelectObject(hPen);
				dc.Ellipse(v[k].rct); 
				s.Format(L"%d",k); 
				dc.TextOut(v[k].home.x-10,v[k].home.y-15,s);
			}
	}
}

int CCode4D::GetCUN()
{
	int minDistance=99;
	int cun;
	for (int i=1;i<=n;i++)
		if ((!fv[i]) && (minDistance>=v[Sv].sp[i]))
		{
			minDistance=v[Sv].sp[i];
			cun=i;
		}
	return cun;
}

void CCode4D::Dijkstra()
{
	int minDistance=99;
	int cun;

	// initialize
	for (int i=1;i<=n;i++)
	{
		fv[i]=false;
		Pv[i]=-1;
		v[Sv].sp[i]=99;
	}
	v[Sv].sp[Sv]=0;

	// compute	
	int k=0;
	while (k<n)
	{
		cun=GetCUN();
		fv[cun]=true;
		for (int i=1;i<=n;i++)
			if ((!fv[i]) && (v[cun].wt[i]>0))
				if (v[Sv].sp[i]>v[Sv].sp[cun]
						+v[cun].wt[i])
				{
					v[Sv].sp[i]=v[Sv].sp[cun]
						+v[cun].wt[i];
					Pv[i]=cun;
				}
		k++;
	}
}

void CCode4D::DrawPath()
{ 
	CClientDC dc(this);
	int w,i,u,r,dummy[NMax+1];

	// display the source			
	dc.SelectObject(pBus[1+cColor%10]); cColor++;	
	w=Dv;	Bus[nBus].nNodes=0; Bus[nBus].Path[1]=Sv;
	for (i=1;i<=n;i++)
	{
		u=Pv[w];
		if (u!=-1)
		{
			r=++Bus[nBus].nNodes; dummy[r]=w;
			w=u;
		}
		dummy[r+1]=Sv;
	}
	r=++Bus[nBus].nNodes; Bus[nBus].Path[r]=Dv;

	u=Sv; Bus[nBus].Path[1]=u;
	dc.MoveTo(v[u].home);
	for (i=1;i<=Bus[nBus].nNodes;i++)
	{
		w=dummy[Bus[nBus].nNodes-i+1]; Bus[nBus].Path[i]=w;
		dc.LineTo(v[w].home);
		u=w;
	}
	w=Dv; Bus[nBus].Path[i]=w;
	dc.LineTo(v[w].home);
}

void CCode4D::OnLButtonDown(UINT nFlags,CPoint pt)
{
	CClientDC dc(this);
	CString s;
	dc.SelectObject(pBus[1]); dc.SelectObject(fArial);
	for (int k=1;k<=n;k++)
		if (v[k].rct.PtInRect(pt))
		{
			LineFlag++;
			if (LineFlag==1 && k<=Ny)
			{
				Sv=k; dc.Rectangle(v[Sv].rct);
				s.Format(L"%d",Sv); 
				dc.TextOut(v[Sv].home.x-15,
					v[Sv].home.y-18,s);
			}
			if (LineFlag==2 && (k>Ny && k<=2*Ny))
			{
				Dv=k; 
				dc.Rectangle(v[Dv].rct);
				Dijkstra();
				if (v[Sv].sp[Dv]!=99)
				{
					nBus++; DrawPath(); ShowTable(); 
					Initialize();
					s.Format(L"%d",Dv);
					dc.TextOut(v[Dv].home.x-15,
						v[Dv].home.y-18,s);
				}
				else
				{
					LineFlag=0;
					dc.SelectObject(pBus[0]); 
					dc.Rectangle(v[Sv].rct); 
					dc.Rectangle(v[Dv].rct);
					s.Format(L"%d",Sv); 
					dc.TextOut(v[Sv].home.x-15,
						v[Sv].home.y-18,s); 
					s.Format(L"%d",Dv); 
					dc.TextOut(v[Dv].home.x-15,
						
						v[Dv].home.y-18,s);
				}
			}
			if (LineFlag==2)
				LineFlag=0;
		}
}

void CCode4D::ShowTable()
{
	int i,k,w,u;
	CString s,S;
	for (i=1;i<=nBus;i++)
	{
		table.DeleteItem(i-1);	
		s.Format(L"%d",i); table.InsertItem(i-1,s,0);
		s.Format(L"%d",Bus[i].Path[1]); table.SetItemText(i-1,1,s);
		w=Bus[i].nNodes; 
		s.Format(L"%d",Bus[i].Path[w]); table.SetItemText(i-1,2,s);
		u=Bus[i].Path[1]; w=Bus[i].Path[w];
		s.Format(L"%d",v[u].sp[w]); table.SetItemText(i-1,3,s);
		s.Format(L"%d",Bus[i].nNodes); table.SetItemText(i-1,4,s);		
		S="";
		for (k=1;k<=Bus[i].nNodes;k++)
		{
			s.Format(L"%d ",Bus[i].Path[k]); S += s;
		}
		table.SetItemText(i-1,5,S);
	}
}