#include "Code7B.h"
#define PI 3.142

BEGIN_MESSAGE_MAP(CCode7B,CFrameWnd)
	ON_BN_CLICKED(IDC_START,OnStart)
	ON_WM_PAINT()
END_MESSAGE_MAP()

CCode7B::CCode7B()
{
	idc=400;
	Create(NULL,L"Delaunay Triangulation",WS_OVERLAPPEDWINDOW,CRect(0,0,1200,800));
	WHome=CPoint(50,50); WEnd=CPoint(600,600);	
	CHome.x=-10; CHome.y=10; CEnd.x=10; CEnd.y=-10;
	eN.Create(WS_CHILD | WS_VISIBLE | WS_BORDER | SS_CENTER, 
		CRect(CPoint(WHome.x+140,WHome.y-40),CSize(70,25)),this,idc++);
	Start.Create(L"Generate",WS_CHILD | WS_VISIBLE | BS_DEFPUSHBUTTON,
		CRect(CPoint(WHome.x+250,WHome.y-40),CSize(150,25)),this,IDC_START);
	fArial.CreatePointFont(80,L"Arial");
}

void CCode7B::OnPaint()
{
	CPaintDC dc(this);
	CPen pBlack(PS_SOLID,2,RGB(0,0,0));
	dc.SelectObject(pBlack);
	CRect rct=CRect(WHome.x,WHome.y,WEnd.x,WEnd.y); 
	dc.Rectangle(rct);
	dc.SelectObject(fArial);
	dc.TextOutW(WHome.x,WHome.y-35,L"#Nodes (3-20, Default: 15)");
}

void CCode7B::OnStart()
{
	CString s;
	CClientDC dc(this);
	CRect rct;
	int i,rx,ry;
	CPoint Q;
	CBrush bkBrush(RGB(255,255,255));
	rct=CRect(WHome.x+2,WHome.y+2,WEnd.x-2,WEnd.y-2); 
	dc.FillRect(rct,&bkBrush);

	eN.GetWindowText(s); n=_ttoi(s);
	n=((n<3 || n>20)?15:n);
	srand(time(0));
	dc.SelectObject(fArial);
	for (i=1;i<=n;i++)
	{
		rx=WEnd.x-WHome.x-30; ry=WEnd.y-WHome.y-30;
		Q.x=WHome.x+15+rand()%rx; Q.y=WHome.y+15+rand()%ry; P[i]=ConvertWC(Q);
		rct=CRect(Q.x,Q.y,Q.x+5,Q.y+5);
		dc.FillSolidRect(rct,RGB(100,100,100)); dc.SetBkColor(RGB(255,255,255));
		s.Format(L"%d",i); dc.TextOutW(Q.x-10,Q.y-10,s); 
		s.Format(L"%d: %.2lf,%.2lf",i,P[i].x,P[i].y);
		dc.TextOutW(WEnd.x+100,WHome.y+10+20*(i-1),s);
	}
	Triangulate();
}

void CCode7B::Triangulate()
{
	CClientDC dc(this);
	CPoint Q;
	double rDistance;
	int i,j,k,ii,r=1;
	for (i=1;i<=n;i++)
		for (j=i+1;j<=n;j++)
			for (k=j+1;k<=n;k++)
			{
				T[r].flag=1;
				sort(r,i,j,k);
				SLE(r);
				for (ii=1;ii<=n;ii++)
					if (ii!=i && (ii!=j && ii!=k)) 
					{
						rDistance=sqrt(pow(T[r].center.x-P[ii].x,2)+pow(T[r].center.y-P[ii].y,2));
						if (rDistance<=T[r].radius)
						{
							T[r].flag=0; break;
						}
					}
				if (T[r].flag)
				{
					Q=ConvertCW(T[r].a); dc.MoveTo(Q);
					Q=ConvertCW(T[r].b); dc.LineTo(Q);
					Q=ConvertCW(T[r].c); dc.LineTo(Q);
					Q=ConvertCW(T[r].a); dc.LineTo(Q);
				}
				r++;
			}
}

void CCode7B::sort(int r,int u,int v,int z)
{
	double tmp1,w[4];
	int s[4],tmp2,i,k;
	s[1]=u; w[1]=P[u].x;
	s[2]=v; w[2]=P[v].x; 
	s[3]=z; w[3]=P[z].x;
	for (k=1;k<=3;k++)
		for (i=1;i<=2;i++)
			if (w[i]>=w[i+1])
			{
				tmp1=w[i]; w[i]=w[i+1]; w[i+1]=tmp1;
				tmp2=s[i]; s[i]=s[i+1]; s[i+1]=tmp2;
			}					
	T[r].a=P[s[1]]; T[r].b=P[s[2]]; T[r].c=P[s[3]];
}

void CCode7B::SLE(int r)
{
	int i,j,k,nsle=2;
	double a[3][3],b[3],xsle[3],Sum,p;
	a[1][1]=2*(T[r].a.x-T[r].b.x); a[1][2]=2*(T[r].a.y-T[r].b.y);
	a[2][1]=2*(T[r].a.x-T[r].c.x); a[2][2]=2*(T[r].a.y-T[r].c.y);
	b[1]=pow(T[r].a.x,2)+pow(T[r].a.y,2)-pow(T[r].b.x,2)-pow(T[r].b.y,2);
	b[2]=pow(T[r].a.x,2)+pow(T[r].a.y,2)-pow(T[r].c.x,2)-pow(T[r].c.y,2);
   	for (k=1;k<=nsle-1;k++)
       	for (i=k+1;i<=nsle;i++)
		{
			p=a[i][k]/a[k][k];
			for (j=1;j<=nsle;j++)
				a[i][j]-=p*a[k][j];
			b[i] -= p*b[k];
		}
    for (i=nsle;i>=1;i--)
    {
       	Sum=0;
		xsle[i]=0;
		for (j=i;j<=nsle;j++)
           	Sum += a[i][j]*xsle[j];
		xsle[i]=(b[i]-Sum)/a[i][i];
    }
	T[r].center.x=xsle[1]; T[r].center.y=xsle[2];
	T[r].radius=sqrt(pow(T[r].center.x-T[r].a.x,2)+pow(T[r].center.y-T[r].a.y,2));
}

PT CCode7B::ConvertWC(CPoint p)		// converts W to C
{
	double m1,c1,m2,c2;
	PT q;
	m1=(CEnd.x-CHome.x)/((double)(WEnd.x-WHome.x));
	c1=(CHome.x-(double)WHome.x*m1);
	m2=(CEnd.y-CHome.y)/(WEnd.y-WHome.y);	
	c2=(CHome.y-(double)WHome.y*m2);
	q.x=(double)p.x*m1+c1; q.y=(double)p.y*m2+c2;
	return q;
}

CPoint CCode7B::ConvertCW(PT p)		// converts C to W
{
	CPoint px;
	double m1,c1,m2,c2;
	m1=(double)(WEnd.x-WHome.x)/(CEnd.x-CHome.x);	
	c1=(double)(WHome.x-CHome.x*m1);
	m2=(double)(WEnd.y-WHome.y)/(CEnd.y-CHome.y);	
	c2=(double)(WHome.y-CHome.y*m2);
	px.x=p.x*m1+c1; px.y=p.y*m2+c2;
	return px;
}
