// code8B.cpp: task scheduling
#include "Code8B.h"

BEGIN_MESSAGE_MAP(CCode8B,CFrameWnd)
	ON_WM_PAINT() 
	ON_WM_LBUTTONDOWN()
	ON_WM_RBUTTONDOWN() 
    ON_BN_CLICKED(IDC_SCHEDULE,OnCompute)
	ON_BN_CLICKED(IDC_NGRAPH,OnNGraph)
END_MESSAGE_MAP()

CCode8B::CCode8B()
{
	WHome=CPoint(30,20); WEnd=CPoint(700,500);
	CHome.x=-8; CHome.y=10; CEnd.x=8; CEnd.y=-10;
	Create(NULL,L"Multiprocessor Task Scheduling",
		WS_OVERLAPPEDWINDOW,CRect(0,0,1000,800));
	TSbutton.Create(L"Task Scheduler",WS_CHILD | WS_VISIBLE | BS_DEFPUSHBUTTON,
		CRect(CPoint(WHome.x+30,WEnd.y+20),CSize(150,30)),this,IDC_SCHEDULE);
	bNGraph.Create(L"New Graph",WS_CHILD | WS_VISIBLE | BS_DEFPUSHBUTTON,
		CRect(CPoint(WHome.x+200,WEnd.y+20),CSize(150,30)),this,IDC_NGRAPH);
	fCourier.CreatePointFont(60,L"Courier");
	fTimes.CreatePointFont(80,L"Times");
	BgColor=RGB(255,255,255);
	TextColor=RGB(100,100,100);
	Initialize();
}

void CCode8B::OnNGraph()
{
	Initialize(); Invalidate();
}

void CCode8B::Initialize()
{
	n=0; RButtonFlag=0;
	for (int k=1;k<=nPE;k++)
    {
		PE[k].v[0]=0;
		PE[k].prt=0;
		PE[k].pel=0;
		PE[k].WHome=CPoint(WHome.x+30,WEnd.y+70+(k-1)*30);
    }
	table.DestroyWindow();
}

void CCode8B::OnPaint()
{
	CPaintDC dc(this);
	CRect rct;
	CString s;
	CPen pDark(PS_SOLID,3,RGB(0,0,0));

	rct=CRect(WHome,WEnd);
	dc.SelectObject(&pDark);
	dc.Rectangle(rct);
	dc.SetTextColor(RGB(255,255,255));
	for (int k=1;k<=nPE;k++)
	{
		rct=CRect(PE[k].WHome,CSize(25,25));
		dc.FillSolidRect(&rct,RGB(100,100,100));
		s.Format(L"%d",k); dc.TextOut(PE[k].WHome.x+8,PE[k].WHome.y+5,s);
	}
	dc.SetBkColor(BgColor);
	dc.TextOutW(WHome.x+40,WEnd.y+20,
		L"left click to draw a node, right click on nodes to draw an edge");
}

void CCode8B::OnLButtonDown(UINT nFlags,CPoint pt)
{
	if (CRect(WHome,WEnd).PtInRect(pt))
		if (n<=N)
		{
			n++;
			v[n].WHome=pt;
			v[n].pre[0]=0;
			v[n].ast=0;
			v[n].act=0;
			v[n].sta=0;		// set Node status to inactive
			v[n].rct=CRect(CPoint(v[n].WHome),CSize(35,25));
			DrawNode(n);
		}
}

void CCode8B::OnRButtonDown(UINT nFlags,CPoint pt)
{
	int i,r;
	srand(time(0));
	for (i=1;i<=n;i++)
		if (v[i].rct.PtInRect(pt))
		{
			RButtonFlag++;
			if (RButtonFlag==1)
				pt1=i;
			if (RButtonFlag==2)
			{
				pt2=i;
				RButtonFlag=0;
				r=++v[pt2].pre[0];
				v[pt2].pre[r]=pt1;
				v[pt2].preCom[r]=1+rand()%5;
				DrawLine();
			}
		}
}

void CCode8B::OnCompute()
{
	PMM();				// sort the nodes according to their colevel values
	PreScheduler();		// initialize each PE with 1st Node
    Scheduler();		// the scheduler
	ShowTable();		// display task information
}

void CCode8B:: ShowTable()
{
	int i,j;    
	table.DestroyWindow();
	table.Create(WS_VISIBLE | WS_CHILD | WS_BORDER | LVS_REPORT
        | LVS_NOSORTHEADER,CRect(WEnd.x+20,WHome.y,WEnd.x+250,WEnd.y),this,IDC_table);
	CString s,S[nFIELDS+1]={"Task","PE","Len","AST","CT","Pre","pmm","Order"};
	table.DeleteAllItems();	
	for (j=0;j<=nFIELDS;j++)
		table.InsertColumn(j,S[j],LVCFMT_CENTER,45);
 	for (i=0;i<=n;i++)
		if (i+1<=n && i+1<=N)
		{
			s.Format(L"%d",i+1);
			table.InsertItem(i,s,0);
			s.Format(L"%d",v[i+1].aPE); table.SetItemText(i,1,s);
			s.Format(L"%d",v[i+1].len); table.SetItemText(i,2,s);
			s.Format(L"%d",v[i+1].ast); table.SetItemText(i,3,s);
			s.Format(L"%d",v[i+1].act); table.SetItemText(i,4,s);
			s.Format(L"%d",v[i+1].pre[0]); table.SetItemText(i,5,s);
			s.Format(L"%d",v[i+1].pmm); table.SetItemText(i,6,s);
			s.Format(L"%d",v[i+1].sort); table.SetItemText(i,7,s);
		}
}

void CCode8B::PMM()	// Sort the tasks according to levels
{
    int i,j,k,r,tmp;
    for (i=1;i<=n;i++)
    {
		v[i].sort=i;
		if (v[i].pre[0]==0)
		   v[i].pmm=0;
		else
		{
		   r=v[i].pre[1];
		   tmp=v[r].len+v[i].preCom[1]+v[r].pmm;
		   for (j=1;j<=v[i].pre[0];j++)
		   {
			   r=v[i].pre[j];
			   tmp=(tmp<v[r].len+v[i].preCom[j]+v[r].pmm)?
				   v[r].len+v[i].preCom[j]+v[r].pmm:tmp;
		   }
		   v[i].pmm=tmp;
		}
		for (k=1;k<=i-1;k++)
		{
			r=v[k].sort;
			if (v[i].pmm<v[r].pmm)
			{
			   for (j=i;j>=k+1;j--)
				   v[j].sort=v[j-1].sort;
			   v[k].sort=i;
			   break;
			}
		}
    }
}

void CCode8B::PreScheduler()
{
    int j,i,u,k,AsPE,r,tmp;
    for (i=1;i<=n;i++)
    {
		u=v[i].sort;
		if (v[u].pre[0]==0)
		{
		   if (i<=nPE)
		   {
			   AsPE=i;
			   v[u].aPE=AsPE;
			   v[u].ast=0;
			   v[u].act=v[u].ast+v[u].len;
			   v[u].sta=1;
			   PE[AsPE].v[0]++;
			   PE[AsPE].v[1]=u;
			   PE[AsPE].prt=v[u].act;
			   PE[AsPE].pel += v[u].len;
		   }
		   else
		   {
			   tmp=PE[1].prt;
			   AsPE=1;
			   for (k=1;k<=nPE;k++)
				   if (tmp>PE[k].prt)
				   {
					   tmp=PE[k].prt;
					   AsPE=k;
				   }
			   v[u].aPE=AsPE;
			   v[u].ast=tmp;
			   v[u].act=v[u].ast+v[u].len;
			   v[u].sta=1;
			   r=++PE[AsPE].v[0];
			   PE[AsPE].v[r]=u;
			   PE[AsPE].prt=v[u].act;
			   PE[AsPE].pel += v[u].len;
		   }
		}
		else
		   if (i<=nPE)
		   {
				AsPE=i;
				v[u].aPE=AsPE;
				r=v[u].pre[1];
				tmp=v[r].act+v[u].preCom[1];
				for (j=1;j<=v[u].pre[0];j++)
				{
					r=v[u].pre[j];
					if (tmp<v[r].act+v[u].preCom[j])
						tmp=v[r].act+v[u].preCom[j];
				}
				v[u].ast=tmp;
				v[u].act=v[u].ast+v[u].len;
				v[u].sta=1;
				PE[AsPE].v[0]++;
				PE[AsPE].v[1]=u;
				PE[AsPE].prt=v[u].act;
				PE[AsPE].pel += v[u].len;
		   }
    }
}

void CCode8B::Scheduler()
{
	CClientDC dc(this);
	CString s;
	CRect rct;
	int u,i,j,k,r,m,AsPE;
	int HiDom,HiDomIndex,HiDomPE,LoDom,LoDomIndex,tmp;
	CPen penGray(PS_SOLID,1,TextColor);

	dc.SelectObject(penGray);
    for (j=1;j<=n;j++)
    {
		u=v[j].sort;
		if (!v[u].sta)
		{
			// calculate hrt
			HiDomIndex=1;
			HiDom=v[u].pre[HiDomIndex];
			v[u].hrt=v[HiDom].act+v[u].preCom[HiDomIndex];
			for (i=1;i<=v[u].pre[0];i++)
			{
				r=v[u].pre[i];
				if (v[u].hrt<v[r].act+v[u].preCom[i])
				{
					v[u].hrt=v[r].act+v[u].preCom[i];
					HiDom=r;HiDomIndex=i;
				}
			}
			// calculate lrt
			if (v[u].pre[0]==1)
				v[u].lrt=v[u].hrt;
			else
			{
				LoDomIndex=((HiDomIndex==1)?2:1);
				LoDom=v[u].pre[LoDomIndex];
				v[u].lrt=v[LoDom].act+v[u].preCom[LoDomIndex];
				for (i=1;i<=v[u].pre[0];i++)
					if (i!=HiDomIndex)
					{
					   r=v[u].pre[i];
					   if (v[u].lrt<v[r].act+v[u].preCom[i])
						   v[u].lrt=v[r].act+v[u].preCom[i];
					}
			}	
			HiDomPE=v[HiDom].aPE;
			if (PE[HiDomPE].prt<=v[u].hrt)
			{
				AsPE=HiDomPE;
				v[u].aPE=AsPE;
				if (v[u].pre[0]==1)
					v[u].ast=PE[HiDomPE].prt;
				else		      
					v[u].ast=((PE[HiDomPE].prt>=v[u].lrt)?PE[HiDomPE].prt:v[u].lrt);
				v[u].act=v[u].ast+v[u].len;
				v[u].sta=1;
				PE[AsPE].prt=v[u].act;
				r=++PE[AsPE].v[0];
				PE[AsPE].v[r]=u;
				PE[AsPE].pel+=v[u].len;
			}
			else
			{
				AsPE=((HiDomPE==1)?2:1);
				tmp=PE[AsPE].pel;
				for (k=1;k<=nPE;k++)
					if (k!=HiDomPE)
						if (tmp>PE[k].pel)
						{
							AsPE=k;
							tmp=PE[k].pel;
						}
				v[u].aPE=AsPE;
				v[u].ast=((PE[AsPE].prt>=v[u].hrt)?PE[AsPE].prt:v[u].hrt);
				v[u].act=v[u].ast+v[u].len;
				v[u].sta=1;
				PE[AsPE].prt=v[u].act;
				r=++PE[AsPE].v[0];
				PE[AsPE].v[r]=u;
				PE[AsPE].pel += v[u].len;
			}
		}
    }
	dc.SelectObject(fCourier); dc.SetTextColor(TextColor);
	for (i=1;i<=n;i++)
	{
		m=v[i].aPE;
		v[i].GHome=CPoint(30+PE[m].WHome.x+20*v[i].ast,PE[m].WHome.y);
		rct=CRect(v[i].GHome.x,v[i].GHome.y,v[i].GHome.x+20*v[i].len,v[i].GHome.y+25);
		dc.Rectangle(rct);
		s.Format(L"%d",i); dc.TextOut(2+v[i].GHome.x,5+v[i].GHome.y,s);
	}
}

void CCode8B::DrawNode(int u)
{	
	CClientDC dc(this);
	CString s;
	CPen pBlack(PS_SOLID,2,RGB(0,0,0));
	dc.SelectObject(pBlack);			
	dc.Rectangle(v[n].rct);
	v[n].len=1+rand()%5;
	s.Format(L"%d:%d",n,v[n].len);
	dc.TextOut(v[n].WHome.x+3,v[n].WHome.y+3,s);
}

void CCode8B::DrawLine()
{	
	CClientDC dc(this);
	double theta;
	CString s;
	PT a,b,c,d;
	CPoint A,B,C,D,mPoint;
	dc.MoveTo(CPoint(v[pt1].WHome));
	dc.LineTo(CPoint(v[pt2].WHome));
	v[pt1].CHome=ConvertWC(v[pt1].WHome);
	v[pt2].CHome=ConvertWC(v[pt2].WHome);

	// Draw arrow
	A=CPoint((v[pt1].WHome.x+3*v[pt2].WHome.x)/4,(v[pt1].WHome.y+3*v[pt2].WHome.y)/4);
	a=ConvertWC(A);
	if (v[pt1].CHome.x<=v[pt2].CHome.x && v[pt1].CHome.y<=v[pt2].CHome.y)
	{
		theta=atan((v[pt2].CHome.y-v[pt1].CHome.y)/(v[pt2].CHome.x-v[pt1].CHome.x));
		b.x=a.x-L*sin(theta); b.y=a.y+L*cos(theta); B=ConvertCW(b);
		c.x=a.x+L*cos(theta); c.y=a.y+L*sin(theta); C=ConvertCW(c);
		d.x=a.x+L*sin(theta); d.y=a.y-L*cos(theta); D=ConvertCW(d);
	}
	if (v[pt1].CHome.x>=v[pt2].CHome.x && v[pt1].CHome.y>=v[pt2].CHome.y)
	{
		theta=atan((v[pt1].CHome.y-v[pt2].CHome.y)/(v[pt1].CHome.x-v[pt2].CHome.x));
		b.x=a.x-L*sin(theta); b.y=a.y+L*cos(theta); B=ConvertCW(b);
		c.x=a.x-L*cos(theta); c.y=a.y-L*sin(theta); C=ConvertCW(c);
		d.x=a.x+L*sin(theta); d.y=a.y-L*cos(theta); D=ConvertCW(d);
	}
	if (v[pt1].CHome.x>=v[pt2].CHome.x && v[pt1].CHome.y<=v[pt2].CHome.y)
	{
		theta=atan((v[pt2].CHome.y-v[pt1].CHome.y)/(v[pt1].CHome.x-v[pt2].CHome.x));
		b.x=a.x+L*sin(theta); b.y=a.y+L*cos(theta); B=ConvertCW(b);
		c.x=a.x-L*cos(theta); c.y=a.y+L*sin(theta); C=ConvertCW(c);
		d.x=a.x-L*sin(theta); d.y=a.y-L*cos(theta); D=ConvertCW(d);
	}
	if (v[pt1].CHome.x<=v[pt2].CHome.x && v[pt1].CHome.y>=v[pt2].CHome.y)
	{
		theta=atan((v[pt1].CHome.y-v[pt2].CHome.y)/(v[pt2].CHome.x-v[pt1].CHome.x));
		b.x=a.x+L*sin(theta); b.y=a.y+L*cos(theta); B=ConvertCW(b);
		c.x=a.x+L*cos(theta); c.y=a.y-L*sin(theta); C=ConvertCW(c);
		d.x=a.x-L*sin(theta); d.y=a.y-L*cos(theta); D=ConvertCW(d);
	}
	dc.MoveTo(B); dc.LineTo(C); dc.LineTo(D);
	int u,w,r;
	r=v[pt2].pre[0];
	u=(v[pt1].WHome.x+v[pt2].WHome.x)/2; 
	w=(v[pt1].WHome.y+v[pt2].WHome.y)/2;
	dc.SelectObject(fTimes);
	s.Format(L"%d",v[pt2].preCom[r]); dc.TextOut(u,w,s);
}

CPoint CCode8B::ConvertCW(PT p)		// converts C to W
{
	CPoint px;
	double m1,c1,m2,c2;
	m1=(double)(WEnd.x-WHome.x)/(CEnd.x-CHome.x);	
	c1=(double)(WHome.x-CHome.x*m1);
	m2=(double)(WEnd.y-WHome.y)/(CEnd.y-CHome.y);	
	c2=(double)(WHome.y-CHome.y*m2);
	px.x=p.x*m1+c1; px.y=p.y*m2+c2;
	return px;
}

PT CCode8B::ConvertWC(CPoint p)		// converts W to C
{
	double m1,c1,m2,c2;
	PT q;
	m1=(CEnd.x-CHome.x)/((double)(WEnd.x-WHome.x));
	c1=(CHome.x-(double)WHome.x*m1);
	m2=(CEnd.y-CHome.y)/(WEnd.y-WHome.y);	
	c2=(CHome.y-(double)WHome.y*m2);
	q.x=(double)p.x*m1+c1; q.y=(double)p.y*m2+c2;
	return q;
}
